# -*- coding: utf-8 -*-
################################################################################
#
# ROSA Crypto
#
# Copyright (c) 2016-2018, LLC "NTC IT ROSA"
# License: BSD
# Authors:
#     Michl Voznesensky <m.voznesensky@rosalinux.ru>
#
# This file is a part of ROSA Crypto
#
# All rights reserved.
#
#  Redistribution and use in source and binary forms, with or without
#  modification, are permitted provided that the following conditions are
#  met:
#
#  * Redistributions of source code must retain the above copyright
#    notice, this list of conditions and the following disclaimer.
#  * Redistributions in binary form must reproduce the above
#    copyright notice, this list of conditions and the following disclaimer
#    in the documentation and/or other materials provided with the
#    distribution.
#  * Neither the name of the LLC "NTC IT ROSA" nor the names of its
#    contributors may be used to endorse or promote products derived from
#    this software without specific prior written permission.
#
#  THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS
#  "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT
#  LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR
#  A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT
#  OWNER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
#  SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT
#  LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE,
#  DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY
#  THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
#  (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE
#  OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
#
################################################################################

import time
import os.path
import subprocess
from transliterate import translit
#translit nedeed for vefify sign
#files with the name Cyrillic characters are not available to take certificate
import re
import datetime
from rosa_crypto_tool.errors import *
from PyQt5.QtCore import QUrl, Qt
from PyQt5.QtGui import QDesktopServices
from PyQt5.QtWidgets import QMessageBox, QCheckBox
from pyudev import Context, Monitor
from pyudev.pyqt5 import MonitorObserver
from configparser import ConfigParser

UTILS_CRYPT = ('csptest', 'certmgr', 'cryptcp')
CONFIG_FILE = os.path.expanduser("~")+"/.config/rosa-crypto-tool.cf"
NOASK = "-nochain" # Not ask - Do you want to use this certificate ([Y]es, [N]o, [C]ancel)?

class RosaApp(object):
	""" Logic of the program """
	
	def __init__(self, app_ui):
		self._ui = app_ui #main window
		
		# Monitoring usb Token
		# monitor started after first run checkProgramm
		self.context = Context() # all devices in system
		self.monitor = Monitor.from_netlink(self.context) # synchronous monitoring
		self.monitor.filter_by(subsystem = 'usb') # monitoring usb devices
		observer = MonitorObserver(self.monitor, self._ui) # asynchronous monitoring for devices
		observer.deviceEvent.connect(self.__detectToken)
		
		# Initialization config file
		self.user_preference = ConfigParser()
		if os.path.isfile(CONFIG_FILE):
			self.user_preference.read(CONFIG_FILE)
		else:
			self.user_preference.add_section("preference")
			self.user_preference.set("preference", "no_show", "False")
			self.user_preference.set("preference", "last_sign", "0")
			self.user_preference.set("preference", "last_decrypt", "0")
			self.user_preference.set("preference", "last_encrypt", "0")
			with open(CONFIG_FILE, "w") as file_cf:
				self.user_preference.write(file_cf)
		
		self.checkProgramm(True)
		
		# Create bound
		self._ui.exit_act.triggered.connect(exit)
		
		self._ui.help_act.triggered.connect(self.openHelp)
		
		self._ui.check_comp_act.triggered.connect(self.checkProgramm)
		
		self._ui.sign.btn_sign.clicked.connect(self.signCreate)
		self._ui.verify_sign.btn_verify_sign.clicked.connect(self.verifySign)
		self._ui.encrypt.btn_encrypt.clicked.connect(self.encryptFile)
		self._ui.decrypt.btn_decrypt.clicked.connect(self.decryptFile)
	
	def checkProgramm(self, first_run = False):
		""" This checks all the necessary components for the successful operation of the program.
		If something is missing you will see the corresponding message """
		
		insert_token = False #check insert token on computer
		self.valid_lic = True #valid license CryptoPro
		self.cp_ver = None #CryptoPro version
		
		if not first_run:
			map(lambda field: field.setEnabled(True), self._ui.list_off_all)
			self._ui.note_mess.clear()
			self._ui.sign.list_avail_cert.clear()
			self._ui.encrypt.list_avail_cert.clear()
			self._ui.decrypt.list_avail_cert.clear()
		
		messages = []
		
		#check installed CryptoPro phase 1
		if os.path.isdir('/opt/cprocsp/bin/amd64/'):
			self.dir_crypt = '/opt/cprocsp/bin/amd64/' #directory CryptoPro
		elif os.path.isdir('/opt/cprocsp/bin/ia32/'):
			self.dir_crypt = '/opt/cprocsp/bin/ia32/'
		else: self.dir_crypt = None
		
		if self.dir_crypt is None:
			map(lambda field: field.setEnabled(False), self._ui.list_off_all) #disable almost all components
			messages.append("crypto")
		else:
			#check installed CryptoPro phase 2
			list_utils_crypt = os.listdir(self.dir_crypt)
			check_utils_crypt = map(lambda util: util in list_utils_crypt, UTILS_CRYPT)
			if False in check_utils_crypt:
				map(lambda field: field.setEnabled(False), self._ui.list_off_all)
				messages.append("crypto")
			else:
				#check CryptoPro license
				cp_lic_out, cp_lic_err = subprocess.Popen("%scpconfig -license -view" %self.dir_crypt.replace("bin", "sbin"), stdout=subprocess.PIPE, stderr=subprocess.PIPE , shell=True).communicate()
				if "3" in cp_lic_err.decode():
					self.valid_lic = False
					map(lambda field: field.setEnabled(False), self._ui.list_off_lic)
					messages.append("license")
				else:
					self.cp_ver = int(re.findall("\d", cp_lic_out.decode())[0])
				
				#chek token
				if self.__listToken():
					insert_token = True
					
					#check running PCSCD (needed for token)
					subprocess.call("%scsptest -keyset -enum_cont -verifyc -fq -uniq" %self.dir_crypt, shell=True) #check, automated run pcscd
					run_pcscd = subprocess.call("pidof pcscd", shell=True)
					if run_pcscd != 0: messages.append("pcsc")
				else:
					messages.append("container")
				
				# Start new flow for checker USB device
				if first_run:
					self.monitor.start()
				
				# Can use subprocess.check_output, but this function work on Python since 2.7
				# To work on python 2.6 uses the old style
				info_cert = subprocess.Popen("%scsptest -keyset -enum_cont -verifyc -fq -uniq" %self.dir_crypt, stdout=subprocess.PIPE, shell=True).communicate()[0]
				self.name_cert = self.__parseConsole(info_cert.decode(), "\\\\")
				if not self.name_cert:
					map(lambda field: field.setEnabled(False), self._ui.list_off_container) #disable components linked to signature
					messages.append("cntnr_missing")
				else:
					
					if insert_token:
						self._ui.stat_prog_icon.setPixmap(self._ui.prog_icon_green)
						self._ui.stat_prog_text.setText(_("Token(s) %s is connected") %(', '.join(self.__listToken())))
					
					# Fill drop down list and set default items
					self._ui.sign.list_avail_cert.addItems(self.name_cert)
					self.__setIndex(self._ui.sign, self.user_preference.get("preference", "last_sign"))
					self._ui.decrypt.list_avail_cert.addItems(self.name_cert)
					self.__setIndex(self._ui.decrypt, self.user_preference.get("preference", "last_decrypt"))
				
				#if cert_all empty then triggered exception
				cert_all, cert_err = subprocess.Popen("%scertmgr -list" %self.dir_crypt, stdout=subprocess.PIPE, stderr=subprocess.PIPE, shell=True).communicate()
				self.dict_subject = dict() #empty dictionary for storage the names and hashes certificates
				if "Empty certificate list\n" not in cert_err.decode():
					subject_all = self.__parseConsole(cert_all.decode(), "Subject")
					sha_all = self.__parseConsole(cert_all.decode(), "SHA1")
					self.dict_subject = dict(zip(subject_all, sha_all))
					self._ui.encrypt.list_avail_cert.addItems(sorted(self.dict_subject))
					self.__setIndex(self._ui.encrypt, self.user_preference.get("preference", "last_encrypt"))
				else:
					self._ui.encrypt.ability_visible = False
		
		self.__alertMessage(messages, first_mess = first_run)
	
	def openHelp(self):
		QDesktopServices.openUrl(QUrl("/usr/share/doc/rosa-crypto-tool/help.pdf")) #open help in default programm
	
	def infoCert(self, where_cert, mode):
		""" Generates information about the certificate.
		When creating a signature file, certificate is taken from Rutoken.
		When verifying a signature - from a file """
		
		if where_cert == "container":
			status_cert = subprocess.Popen("%scertmgr -list -cont '%s'" %(self.dir_crypt, mode.list_avail_cert.currentText()), stdout=subprocess.PIPE, shell=True).communicate()[0]
		elif where_cert == "file":
			status_cert = subprocess.Popen([self.dir_crypt + "certmgr", "-list", "-file", mode.file_edit.text()], stdout=subprocess.PIPE).communicate()[0]
		elif where_cert == "store":
			status_cert = subprocess.Popen("%scertmgr -list -thumbprint '%s'" %(self.dir_crypt, self.dict_subject[unicode(mode.list_avail_cert.currentText())]), stdout=subprocess.PIPE, shell=True).communicate()[0]
		
		status_cert = status_cert.decode()
		row_valid_cert = self.__parseConsole(status_cert, "Not valid after")[0] #need to check valid certificate
		self.date_valid_cert = datetime.datetime.strptime(row_valid_cert, "%d/%m/%Y %H:%M:%S UTC")
		self.sha = self.__parseConsole(status_cert, "SHA1")[0]
		
		self.out_status_cert = _("Information about the certificate:\n") +\
							_("- Issuer: ") + self.__parseConsole(status_cert, "Issuer")[0] + "\n" +\
							_("- Subject: ") + self.__parseConsole(status_cert, "Subject")[0] + "\n" +\
							_("- Serial: ") + self.__parseConsole(status_cert, "Serial")[0] + "\n" +\
							_("- Valid: from ") + self.__parseConsole(status_cert, "Not valid before")[0] + _(" to ") + row_valid_cert + "\n"
		
		if self.cp_ver == 4:
			gost_sig = self.__parseConsole(status_cert, "Signature Algorithm")[0]
			self.gost_pubkey = self.__parseConsole(status_cert, "PublicKey Algorithm")[0]
			
			self.out_status_cert += _("- Signature Algorithm: ") + gost_sig + "\n" +\
									_("- PublicKey Algorithm: ") + self.gost_pubkey
		
	
	def signCreate(self):
		""" Create sign for file with the extension .sig """
		
		if self._ui.sign.file_edit.text():
			self.infoCert("container", self._ui.sign)
			
			new_name = translit(self._ui.sign.file_name, reversed=True) + ".sig"
			
			#console_cmd = '%scryptcp -dir "%s" -sign -thumbprint "%s" "%s" -der "%s" "%s"' %(self.dir_crypt, self._ui.sign.currently_dir, self.sha, NOASK, self._ui.sign.path_name, self._ui.sign.currently_dir + new_name)
			console_cmd = [self.dir_crypt+'cryptcp', '-dir', self._ui.sign.currently_dir, '-sign', '-thumbprint', self.sha, NOASK, '-der', self._ui.sign.path_name.__str__().encode('utf-8'), os.path.join(self._ui.sign.currently_dir, new_name.__str__())]
			raw_result_sign = subprocess.Popen(console_cmd, stdout=subprocess.PIPE).communicate()[0]
			
			result_sign = searchError(raw_result_sign.decode()) #return tuple (code error, message error)
			
			# Signature successful
			if result_sign[0] == "0":
				out_status_sig = _('The file "%s" has been signed.\nCreated a new signature file "%s"') %(self._ui.sign.file_name, new_name)
				self.__outMessage(out_status_sig, self._ui.sign.icon_success, self._ui.sign, True)
			# Certificates not found
			elif result_sign[0] == "0x2000012d":
				# Dont't change the single quotation marks in the name of the container, else slashes must be doubled
				raw_result_inst_cert = subprocess.Popen("%scertmgr -inst -cont '%s' -store uMy" %(self.dir_crypt, self._ui.sign.list_avail_cert.currentText()), stdout=subprocess.PIPE, stderr=subprocess.PIPE, shell=True)
				out_result_inst_cert, err_result_inst_cert = raw_result_inst_cert.communicate()
				if err_result_inst_cert.decode():
					out_status_sig = _('\nError: ') + err_result_inst_cert.decode()
					self.__outMessage(out_status_sig, self._ui.sign.icon_atten, self._ui.sign, False)
				else:
					subject_new = self.__parseConsole(out_result_inst_cert.decode(), "Subject")
					sha_new = self.__parseConsole(out_result_inst_cert.decode(), "SHA1")[0]
					self._ui.encrypt.list_avail_cert.insertItems(0, subject_new)
					self.dict_subject[subject_new[0]] = sha_new
					self._ui.encrypt.ability_visible = True
					self.signCreate()
			else:
				out_status_sig = _('Failed to sign file "%s". %s\nError code: %s.') %(self._ui.sign.file_name, result_sign[1], result_sign[0])
				self.__outMessage(out_status_sig, self._ui.sign.icon_failed, self._ui.sign, False)
			
			self._ui.sign.btn_details.setEnabled(True)

	def verifySign(self):
		""" Verify file with the extension .sig """
		
		if self._ui.verify_sign.file_edit.text():
			os.chdir(self._ui.verify_sign.currently_dir)
			#If you do not change directory to the directory
			#of the current file, the certificate file is not found
			
			try:
				#check options
				if self._ui.verify_sign.verify_split.checkState():
					if '.sig' in self._ui.verify_sign.file_name:
						new_name = self._ui.verify_sign.file_name[:self._ui.verify_sign.file_name.rindex(".")] #separate extention .sig
					else:
						new_name = "new_" + self._ui.verify_sign.file_name
					
					#console_cmd = '%scryptcp -verify "%s" "%s" -f "%s" "%s"' %(self.dir_crypt, self._ui.verify_sign.path_name, new_name, self._ui.verify_sign.file_name, NOASK)
					console_cmd = [self.dir_crypt+'cryptcp', '-verify', self._ui.verify_sign.path_name.__str__().encode('utf-8'), new_name, '-f', self._ui.verify_sign.file_name, NOASK]
					raw_result_verify = subprocess.Popen(console_cmd, stdout=subprocess.PIPE).communicate()[0]
					
					result_verify = searchError(raw_result_verify.decode())
					
					if result_verify[0] == "0":
						out_status_verify_sign = _('The signature file "%s" is checked.\nThe source file is separated from the signature and saved with the name "%s"') %(self._ui.verify_sign.file_name, new_name)
					else:
						raise Exception(result_verify[0], result_verify[1]) #separate because except return tuple
				else:
					#console_cmd = '%scryptcp -verify "%s" -f "%s" "%s"' %(self.dir_crypt, self._ui.verify_sign.path_name, self._ui.verify_sign.file_name, NOASK)
					console_cmd = [self.dir_crypt+'cryptcp', '-verify', self._ui.verify_sign.path_name.__str__().encode('utf-8'), '-f', self._ui.verify_sign.file_name, NOASK]
					raw_result_verify = subprocess.Popen(console_cmd, stdout=subprocess.PIPE).communicate()[0]
					
					result_verify = searchError(raw_result_verify.decode())
					
					if result_verify[0] == "0":
						out_status_verify_sign = _('The signature file "%s" is checked.') %self._ui.verify_sign.file_name
					else:
						raise Exception(result_verify[0], result_verify[1])
			
			except Exception as inst:
				if inst[0] == "0x2000012d":
					out_status_verify_sign = _("This file is not a signature")
					self.__outMessage(out_status_verify_sign, self._ui.verify_sign.icon_failed, self._ui.verify_sign, False)
				else:
					out_status_verify_sign = _('Failed to verify file "%s". %s\nError code: %s.') %(self._ui.verify_sign.file_name, inst[1], inst[0])
					self.__outMessage(out_status_verify_sign, self._ui.verify_sign.icon_failed, self._ui.verify_sign, False)
			
			else:
				if self._ui.verify_sign.verify_install.checkState():
					#result_inst_cert = subprocess.Popen("%scertmgr -inst -file '%s' -store uMy" %(self.dir_crypt, self._ui.verify_sign.file_name), stdout=subprocess.PIPE, shell=True).communicate()[0]
					result_inst_cert = subprocess.Popen([self.dir_crypt+'certmgr', '-inst', '-file', self._ui.verify_sign.file_name.__str__().encode('utf-8'), '-store', 'uMy'], stdout=subprocess.PIPE).communicate()[0]
					result_inst_cert = result_inst_cert.decode()
					print(result_inst_cert)
					out_status_verify_sign += _("\nThe certificate is installed.")
					subject_inst_cert = self.__parseConsole(result_inst_cert, "Subject")[0]
					#check installed already this certificate or not
					if subject_inst_cert not in self.dict_subject:
						sha_inst_cert = self.__parseConsole(result_inst_cert, "SHA1")[0]
						self.dict_subject[subject_inst_cert] = sha_inst_cert
						self._ui.encrypt.list_avail_cert.insertItem(0, subject_inst_cert)
						self._ui.encrypt.ability_visible = True
				
				self.infoCert("file", self._ui.verify_sign)
				self.__outMessage(out_status_verify_sign, self._ui.verify_sign.icon_success, self._ui.verify_sign, True)
			
			finally:
				self._ui.verify_sign.btn_details.setEnabled(True)
	
	def encryptFile(self):
		""" Create encrypted file with the extension .msg """
		
		if self._ui.encrypt.file_edit.text():
			os.chdir(self._ui.encrypt.currently_dir)
			
			self.infoCert("store", self._ui.encrypt)
			
			new_name = translit(self._ui.encrypt.file_name, reversed=True) + ".enc"
			
			#console_cmd = '%scryptcp -encr -thumbprint "%s" "%s" -der "%s" "%s"' %(self.dir_crypt, self.sha, NOASK, self._ui.encrypt.path_name, self._ui.encrypt.currently_dir + new_name)
			console_cmd = [self.dir_crypt+'cryptcp', '-encr', '-thumbprint', self.sha, NOASK, '-der', self._ui.encrypt.path_name.__str__().encode('utf-8'), os.path.join(self._ui.encrypt.currently_dir, new_name.__str__())]
			raw_result_encrypt = subprocess.Popen(console_cmd, stdout=subprocess.PIPE).communicate()[0]
			
			result_encrypt = searchError(raw_result_encrypt.decode())
			
			if result_encrypt[0] == "0":
				out_status_encrypt_file = _('The file "%s" is encrypted in file "%s"') %(self._ui.encrypt.file_name, new_name)
				self.__outMessage(out_status_encrypt_file , self._ui.encrypt.icon_success, self._ui.encrypt, True)
			else:
				out_status_encrypt = _('Failed to encrypted file "%s". %s\nError code: %s.') %(self._ui.encrypt.file_name, result_encrypt[1], result_encrypt[0])
				self.__outMessage(out_status_encrypt, self._ui.encrypt.icon_failed, self._ui.encrypt, False)
			
			self._ui.encrypt.btn_details.setEnabled(True)
	
	def decryptFile(self):
		""" Decrypted file """
		
		if self._ui.decrypt.file_edit.text():
			os.chdir(self._ui.decrypt.currently_dir)
			self.infoCert("container", self._ui.decrypt)
			
			if '.enc' in self._ui.decrypt.file_name:
				new_name = self._ui.decrypt.file_name[:self._ui.decrypt.file_name.rindex(".")] #separate extention .msg
			else:
				new_name = "new_" + self._ui.decrypt.file_name
			
			#console_cmd = '%scryptcp -decr -thumbprint "%s" "%s" -start "%s" "%s"' %(self.dir_crypt, self.sha, NOASK, self._ui.decrypt.file_name, new_name)
			console_cmd = [self.dir_crypt+'cryptcp', '-decr', '-thumbprint', self.sha, NOASK, '-start', self._ui.decrypt.file_name.__str__().encode('utf-8'), new_name]
			raw_result_decrypt = subprocess.Popen(console_cmd, stdout=subprocess.PIPE).communicate()[0]
			
			result_decrypt = searchError(raw_result_decrypt.decode())
			
			if result_decrypt[0] == "0":
				out_status_decrypt = _('The file "%s" is decrypted in file "%s"') %(self._ui.decrypt.file_name, new_name)
				self.__outMessage(out_status_decrypt, self._ui.decrypt.icon_success, self._ui.decrypt, True)
			elif result_decrypt[0] == "0x2000012d" or result_decrypt[0] == "0x20000136":
				# Dont't change the single quotation marks in the name of the container, else slashes must be doubled
				raw_result_inst_cert = subprocess.Popen("%scertmgr -inst -cont '%s' -store uMy" %(self.dir_crypt, self._ui.decrypt.list_avail_cert.currentText()), stdout=subprocess.PIPE, stderr=subprocess.PIPE, shell=True)
				raw_result_inst_cert = raw_result_inst_cert.decode()
				out_result_inst_cert, err_result_inst_cert = raw_result_inst_cert.communicate()
				out_result_inst_cert = out_result_inst_cert.decode()
				err_result_inst_cert = err_result_inst_cert.decode()
				if err_result_inst_cert:
					out_status_decrypt = _('\nError: ') + err_result_inst_cert
					self.__outMessage(out_status_decrypt, self._ui.decrypt.icon_atten, self._ui.decrypt, False)
				else:
					self._ui.encrypt.list_avail_cert.insertItems(0, self.__parseConsole(out_result_inst_cert, "Subject"))
					self._ui.encrypt.ability_visible = True
					self.decryptFile()
			else:
				out_status_decrypt = _('Failed to decrypted file "%s". %s\nError code: %s.') %(self._ui.decrypt.file_name, result_decrypt[1], result_decrypt[0])
				self.__outMessage(out_status_decrypt, self._ui.decrypt.icon_failed, self._ui.decrypt, False)
			
			self._ui.decrypt.btn_details.setEnabled(True)
	
	def installCert(self):
		""" Installing certificate from token """
		
		index_res = []
		result = map(lambda x: subprocess.call([self.dir_crypt+"certmgr", "-inst", "-cont", x, "-store", "uMy"]), self.name_cert)
		for res in result:
			if res != 0:
				index_res.append(result.index(res))
		if not index_res:
			out_status_install_cert = _("The certificate was installed successfully")
			self.__alertMessage(out_status_install_cert, "Information")
		else:
			out_status_install_cert = _("The installation failed.\nPlease check for the token %s in the computer and try again" %', '.join(map(lambda x: self.name_cert[x], index_res)))
			self.__alertMessage(out_status_install_cert, "Critical")
	
	def __setIndex(self, mode, index):
		""" Set index from previously session selected items """
		
		index = int(index)
		mode.list_avail_cert.setCurrentIndex(index) if mode.list_avail_cert.count() > index else mode.list_avail_cert.setCurrentIndex(0)
	
	def __listToken(self):
		""" Return full list uses Token in computers """
		
		list_token = []
		for device in self.context.list_devices(subsystem='usb'):
			if 'ID_MODEL' in device and 'token' in device['ID_MODEL'].lower():
				list_token.append(device['ID_SERIAL'].replace('_', ' '))
		return list_token
	
	def __detectToken(self, device):
		""" Rechecks the presence of the token
		Partially similar to the code from the function checkProgramm """
		
		#print device.action, device.items() #for debug uses token
		
		messages = []
		insert_token = False
		
		if 'ID_MODEL' in device and 'token' in device['ID_MODEL'].lower():
			if device.action == 'add':
				mess_bar = _("Add %s") %device['ID_SERIAL'].replace('_', ' ')
			elif device.action == 'remove':
				mess_bar = _("Remove %s") %device['ID_SERIAL'].replace('_', ' ')
			
			if self._ui.welcome_text.isVisible():
				self._ui.note_mess.setText("<FONT SIZE = '-1'><BR><BR><BR>%s</FONT>" %mess_bar)
				#if use once repaint then temporary text is not displayed
				self._ui.repaint()
				self._ui.repaint(self._ui.main_widget.rect())
			else:
				self._ui.statusBar().showMessage(mess_bar, 0)
				self._ui.repaint(self._ui.statusBar().rect()) #repaint status bar for display temporary message
			
			#print device.action, device #for debug uses token
			
			if self.__listToken():
				insert_token = True
				
				#check running PCSCD (needed for token)
				subprocess.call("%scsptest -keyset -enum_cont -verifyc -fq -uniq" %self.dir_crypt, shell=True) #check, automated run pcscd
				run_pcscd = subprocess.call("pidof pcscd", shell=True)
				if run_pcscd != 0: messages.append("pcsc")
			else:
				messages.append("container")
			
			time.sleep(1) #need a delay, otherwise the CryptoPro may not have time to see the presence of a token in the computer
			info_cert = subprocess.Popen("%scsptest -keyset -enum_cont -verifyc -fq -uniq" %self.dir_crypt, stdout=subprocess.PIPE, shell=True).communicate()[0]
			self.name_cert = self.__parseConsole(info_cert.decode(), "\\\\")
			if not self.name_cert:
				map(lambda field: field.setEnabled(False), self._ui.list_off_container) #disable components linked to signature
				messages.append("cntnr_missing")
			else:
				map(lambda field: field.setEnabled(True), self._ui.list_off_container)
				
				if not self.valid_lic:
					map(lambda field: field.setEnabled(False), self._ui.list_off_lic)
				
				if insert_token:
					self._ui.stat_prog_icon.setPixmap(self._ui.prog_icon_green)
					self._ui.stat_prog_text.setText(_("Token(s) %s is connected") %(', '.join(self.__listToken())))
				
				self._ui.note_mess.clear()
				self._ui.sign.list_avail_cert.clear()
				self._ui.decrypt.list_avail_cert.clear()
				
				self._ui.sign.list_avail_cert.addItems(self.name_cert)
				self._ui.decrypt.list_avail_cert.addItems(self.name_cert)
			
			self.__alertMessage(messages, first_mess = True)
			self._ui.statusBar().clearMessage()
	
	def __parseConsole(self, cert, field):
		""" Separate output from the console by selecting the rows
		and pulling from it the necessary information """
		
		list_key_word = []
		
		for row in re.findall(r"%s.*" %field, cert):
			# Unusual field in certificate
			if field == "Issuer" or field == "Subject":
				#This parsing basically uses to catch name Subject
				entry = row.find("CN=")+3
				end_row = row[entry:].find(",")
				if end_row >= 0:
					key_word = row[entry:(entry + end_row)]
				else:
					key_word = row[entry:]
			elif field == "Serial":
				key_word = row[row.find("0x")+2:]
			elif field == "SHA1":
				try:
					#work on CryptoPro <= v4 R2
					key_word = row[row.index("0x")+2:]
				except ValueError:
					key_word = row[row.index(":")+2:]
			# Field about container
			elif field == "\\\\":
				key_word = row[row.find("|")+1:]
			# Usual field in certificate
			else:
				key_word = row[row.find(": ")+2:]
			
			try:
				list_key_word.append(key_word.decode('utf-8'))
			except UnicodeDecodeError:
				list_key_word.append(key_word.decode('cp1251'))
		return list_key_word
	
	def __validDate(self):
		""" Check valid certificate and algorithm signature """
		
		message = ""
		#valid certificate
		current_datetime = datetime.datetime.utcnow()
		if self.date_valid_cert < current_datetime:
			message += _(" - no longer valid\n")
		#valid algorithm signature
		if self.cp_ver > 3:
			if '2001' in self.gost_pubkey:
				message += _(" - using outdated signature scheme %s valid until 31 December 2018\n") %self.gost_pubkey
		else:
			message += _(" - using outdated signature scheme because of the use of CryptoPro version below 4.0\n")
		
		if message:
			message = _("This certificate:\n") + message
			self.__alertMessage(message, "Warning", enable_checkbox = True, no_show = self.user_preference.get("preference", "no_show"))
			message = _("\n\nWARNING!\n") + message
		
		return message
	
	def __outMessage(self, message, icon_status, mode, info_cert = False):
		""" Create output message on display about completed operation  """
		
		mode.icon_status_oper.setPixmap(icon_status)
		mode.old_mes_status_oper = message
		mode.mes_status_oper.setText(mode.elided_text.elidedText(message, Qt.ElideRight, mode.parent.width(), Qt.TextWordWrap))
		if info_cert:
			mode.result.setText(message + "\n\n" + self.out_status_cert + self.__validDate())
		else:
			mode.result.setText(message)
	
	def __alertMessage(self, messages, type_mess = "Information", first_mess = False, enable_checkbox = False, no_show = "False"):
		"""
		Create alert message in separate window or at the start of the program
		
		messages - list or one given messages
		type_mess - type messages
		first_mess - pointer to the fact that the message is to be displayed
						in the label note_mess that is when you first start program
		enable_checkbox - enable checkbox for message related __validDate
		no_show - responsible about display out message
		"""
		
		if no_show == "False":
			if isinstance(messages, list):
				out_mess = ""
				if messages:
					for mess in messages:
						if mess == 'crypto':
							out_mess += _("CryptoPro is not installed in the computer.<P>Please install CryptoPro to start working in the program.<P>")
							type_mess = "Critical"
						elif mess == 'container':
							out_mess += _("Token is not installed in the computer.<P>")
							type_mess = "Information"
							self._ui.stat_prog_icon.setPixmap(self._ui.prog_icon_orange)
							self._ui.stat_prog_text.setText(_("Token is not installed in the computer"))
						elif mess == 'cntnr_missing':
							out_mess += _("Not found any container with certificate.<P>Signing and decrypting files is not available.<P>")
							type_mess = "Critical"
							self._ui.stat_prog_icon.setPixmap(self._ui.prog_icon_red)
							self._ui.stat_prog_text.setText(_("Not found any container with certificate"))
						elif mess == 'license':
							out_mess += _("License CryptoPro is not valid<P>")
							type_mess = "Warning"
						elif mess == 'pcsc':
							out_mess += _("PC/SC Smart Card Daemon is not running.<P>If you need working with token please run utility pcscd.<P>")
							type_mess = "Critical"
							self._ui.stat_prog_icon.setPixmap(self._ui.prog_icon_red)
							self._ui.stat_prog_text.setText(_("PC/SC Smart Card Daemon is not running"))
				elif not first_mess:
					out_mess = _("All the necessary components installed and working")
			else:
				out_mess = messages
			
			if not first_mess:
				if type_mess == "Information":
					QMessageBox.information(self._ui, _("Information"), out_mess, QMessageBox.Ok)
				elif type_mess == "Warning":
					warning_mess = QMessageBox(QMessageBox.Warning, _("Warning"), out_mess, QMessageBox.Ok, self._ui)
					if enable_checkbox:
						status_show_mess = QCheckBox(_("Don't show this message box again"))
						status_show_mess.blockSignals(True)
						warning_mess.addButton(status_show_mess, QMessageBox.ResetRole)
					
					out_warning_mess = warning_mess.exec_()
					
					if out_warning_mess == QMessageBox.Ok:
						if enable_checkbox and status_show_mess.checkState():
							self.user_preference.set("preference", "no_show", "True")
							with open(CONFIG_FILE, "w") as file_cf:
								self.user_preference.write(file_cf)
				elif type_mess == "Critical":
					QMessageBox.critical(self._ui, _("Critical"), out_mess, QMessageBox.Ok)
			elif out_mess:
				if type_mess == "Information":
					self._ui.note_mess.setText(_("<FONT SIZE = '-1'><BR><BR><BR><B>Note: </B>%s</FONT>")%out_mess)
				elif type_mess == "Warning":
					self._ui.note_mess.setText(_("<FONT SIZE = '-1'><BR><BR><BR><B>Warning: </B>%s</FONT>")%out_mess)
				elif type_mess == "Critical":
					self._ui.note_mess.setText(_("<FONT SIZE = '-1'><BR><BR><BR><B>Attention: </B>%s</FONT>")%out_mess)
	
	def __del__(self):
		""" Write last selected index in all mods """
		
		self.user_preference.set("preference", "last_sign", self._ui.sign.list_avail_cert.currentIndex())
		self.user_preference.set("preference", "last_decrypt", self._ui.decrypt.list_avail_cert.currentIndex())
		self.user_preference.set("preference", "last_encrypt", self._ui.encrypt.list_avail_cert.currentIndex())
		with open(CONFIG_FILE, "w") as file_cf:
			self.user_preference.write(file_cf)

if __name__ == '__main__':
	
	print("You run the part program rosa crypto tool, please launch the main file")
	
	#for debug subprocess
	#try:
		#subprocess.check_call(console_cmd)
	#except subprocess.CalledProcessError as e:
		#print e.cmd
