# -*- coding: utf-8 -*-
################################################################################
#
# ROSA Crypto
#
# Copyright (c) 2016-2018, LLC "NTC IT ROSA"
# License: BSD
# Authors:
#     Michl Voznesensky <m.voznesensky@rosalinux.ru>
#
# This file is a part of ROSA Crypto
#
# All rights reserved.
#
#  Redistribution and use in source and binary forms, with or without
#  modification, are permitted provided that the following conditions are
#  met:
#
#  * Redistributions of source code must retain the above copyright
#    notice, this list of conditions and the following disclaimer.
#  * Redistributions in binary form must reproduce the above
#    copyright notice, this list of conditions and the following disclaimer
#    in the documentation and/or other materials provided with the
#    distribution.
#  * Neither the name of the LLC "NTC IT ROSA" nor the names of its
#    contributors may be used to endorse or promote products derived from
#    this software without specific prior written permission.
#
#  THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS
#  "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT
#  LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR
#  A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT
#  OWNER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
#  SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT
#  LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE,
#  DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY
#  THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
#  (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE
#  OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
#
################################################################################

import os
import subprocess
from PyQt5.QtCore import *
import magic
#this binding to libmagic almost always include in program "file"
#NOT CONFUSE with another module python-magic sources which placed on GitHub
#homepage this module http://www.darwinsys.com/file/
from PyQt5.QtWidgets import QApplication, QMainWindow, QWidget, \
						QGridLayout, QLayout, QPushButton, QLabel, QLineEdit, \
						QTextEdit, QFileDialog, QAction, QActionGroup, \
						QSizePolicy, QHBoxLayout, QVBoxLayout, QFrame, \
						QComboBox, QCheckBox, QToolButton, QMenu, \
						QProgressBar
from PyQt5.QtGui import QCursor, QIcon, QPixmap, QKeySequence, QFont, QFontMetrics, QDesktopServices

#const size window app
MAX_WIDTH = 1024
MIN_WIDTH = 700
MIN_HEIGHT = 320
MAX_HEIGHT = 600
MAX_ROW_RESULT = 45

PREFIX16 = os.path.dirname(__file__) + '/icon/16x16/' #directory icons
PREFIX32 = os.path.dirname(__file__) + '/icon/32x32/'
PREFIX48 = os.path.dirname(__file__) + '/icon/48x48/'

OUTPUT_PDF = '/tmp/rct_pdf.pdf' #absolut path for storage converted pdf file
OUTPUT_PNG = '/tmp/rct_img.png' #absolut path for storage converted png image

DEVNULL = open(os.devnull, 'wb') #for hide stdout subprocess

class MainWindow(QMainWindow):
	""" Main window """
	
	def __init__(self):
		""" Create main widget (window) """
		
		super(MainWindow, self).__init__()
		
		#Cannot disabled button "Deploy window" because decoration in Fresh KDE or Plasma not correct working
		#self.setWindowFlags(Qt.WindowMinimizeButtonHint) #disable button 'deploy window'
		self.setFixedHeight(MIN_HEIGHT)
		self.setMaximumWidth(MAX_WIDTH)
		
		self.main_widget = QWidget(self)
		self.welcome_text = QLabel(_("\n\n\nWelcome to program ROSA Crypto Tool !\n\nTo get started select the desired mode."))
		self.note_mess = QLabel()
		self.welcome_text.setAlignment(Qt.AlignHCenter)
		self.note_mess.setAlignment(Qt.AlignCenter)
		workspace = QVBoxLayout() #main workspace
		workspace.addWidget(self.welcome_text)
		workspace.addWidget(self.note_mess)
		self.main_widget.setLayout(workspace)
		self.setCentralWidget(self.main_widget)
		
		#Action
		#Can use QIcon.fromTheme, but this function work on Qt since 4.6.4
		
		self.opt_act = QAction(QIcon(PREFIX32 + 'options'), _("&Options"), self)
		self.opt_act.setToolTip(_("See also additional options"))
		
		self.open_act = QAction(QIcon(PREFIX16 + 'open'), _("&Choose File"), self)
		self.open_act.setShortcut(QKeySequence.Open)
		self.open_act.setCheckable(True)
		self.open_act.setToolTip(_("Choose file for sign or verify signature"))
		
		self.exit_act = QAction(QIcon(PREFIX16 + 'exit'), _("&Exit"), self)
		self.exit_act.setShortcut(QKeySequence("Ctrl+Q")) #Qt4 low 4.6.4 is not support QKeySequence.Quit
		
		self.sign_act = QAction(QIcon(PREFIX48 + 'sign'), _("&Sign\n file"), self)
		self.sign_act.setToolTip(_("Sign the selected file"))
		self.sign_act.setCheckable(True)
		
		self.verify_sign_act = QAction(QIcon(PREFIX48 + 'verify_sign'), _("&Verify\n signature"), self)
		self.verify_sign_act.setToolTip(_("Verify a signed file"))
		self.verify_sign_act.setCheckable(True)
		
		self.encrypt_act = QAction(QIcon(PREFIX48 + 'crypt'), _("&Encrypt"), self)
		self.encrypt_act.setToolTip(_("Encrypted file"))
		self.encrypt_act.setCheckable(True)
		
		self.decrypt_act = QAction(QIcon(PREFIX48 + 'crypt'), _("&Descrypt"), self)
		self.decrypt_act.setToolTip(_("Descrypted file"))
		self.decrypt_act.setCheckable(True)
		
		self.check_comp_act = QAction(QIcon(PREFIX16 + 'check'), _("Check component program"), self)
		
		self.help_act = QAction(QIcon(PREFIX16 + 'help'), _("&Help"), self)
		self.help_act.setShortcut(QKeySequence.HelpContents)
		
		self.about_act = QAction(QIcon(PREFIX16 + 'about'), _("&About ROSA Crypto Tool"), self)
		
		#StatusBar
		self.prog_icon_green = QIcon(PREFIX16 + 'green.svg').pixmap(16, 16)
		self.prog_icon_orange = QIcon(PREFIX16 + 'orange.svg').pixmap(16, 16)
		self.prog_icon_red = QIcon(PREFIX16 + 'red.svg').pixmap(16, 16)
		self.stat_prog_icon = QLabel()
		self.stat_prog_text = QLabel()
		self.statusBar().addWidget(self.stat_prog_icon)
		self.statusBar().addWidget(self.stat_prog_text)
		self.statusBar().setVisible(False)
		
		#Menu options
		opt_menu = QMenu(self)
		opt_menu.addAction(self.check_comp_act)
		opt_menu.addSeparator()
		opt_menu.addAction(self.about_act)
		opt_menu.addAction(self.help_act)
		opt_menu.addSeparator()
		opt_menu.addAction(self.exit_act)
		
		#ToolBar
		toolbar = self.addToolBar("Mode")
		toolbar.setMovable(False) #dockable panel
		toolbar.setContextMenuPolicy(Qt.PreventContextMenu) #disable context menu
		toolbar.setIconSize(QSize(32, 32))
		toolbar.setToolButtonStyle(Qt.ToolButtonTextUnderIcon)
		
		options = QToolButton(self)
		options.setToolButtonStyle(Qt.ToolButtonTextUnderIcon)
		options.setPopupMode(QToolButton.InstantPopup)
		options.setDefaultAction(self.opt_act)
		options.setMenu(opt_menu)
		
		spacing_tb = QWidget()
		spacing_tb.setSizePolicy(QSizePolicy.Expanding,QSizePolicy.Preferred)
		
		action_group = QActionGroup(self)
		action_group.addAction(self.sign_act)
		action_group.addAction(self.verify_sign_act)
		action_group.addAction(self.encrypt_act)
		action_group.addAction(self.decrypt_act)
		
		toolbar.addActions(action_group.actions())
		# Due to the fact that the font may be different in different systems.
		# Find the maximum width of all buttons in the toolbar and take the value 91% rounding to a whole.
		dynamic_width = int(max(map(lambda mode: toolbar.widgetForAction(mode).sizeHint().width(), action_group.actions()))*91/100)
		map(lambda mode: toolbar.widgetForAction(mode).setFixedWidth(dynamic_width), action_group.actions()) #.setFixedWidth(95)
		toolbar.addWidget(spacing_tb) #spacing
		toolbar.addWidget(options)
		self.toolbar_height = toolbar.sizeHint().height()
		
		self.about_program = AboutWidget()
		
		self.sign = SignFrame(self.main_widget)
		self.verify_sign = VerifyFrame(self.main_widget)
		self.encrypt = EncryptFrame(self.main_widget)
		self.decrypt = DescryptFrame(self.main_widget)
		self.mess = MessFrame(self.main_widget)
		
		self.list_mode = [self.sign, self.verify_sign, self.encrypt, self.decrypt, self.mess]
		
		map(lambda mode: workspace.addWidget(mode), self.list_mode)
		
		toolbar.actionTriggered.connect(self.switchMode)
		self.about_act.triggered.connect(self.showAbout)
		self.list_off_container = [self.sign_act, self.decrypt_act, self.sign, self.decrypt]
		self.list_off_lic = [self.sign_act, self.sign]
		self.list_off_all = [self.sign_act, self.verify_sign_act, self.encrypt_act, self.decrypt_act,\
							self.sign, self.verify_sign, self.encrypt, self.decrypt]
		
		self.resize(MIN_WIDTH, MIN_HEIGHT) #widht and hight window
		
		self.setWindowTitle(_("ROSA Crypto Tool"))
		self.setWindowIcon(QIcon(PREFIX48 + 'rosa-crypto-tool'))
		self.show()
	
	def showAbout(self):
		current_pos = self.about_program.geometry()
		current_pos.moveCenter(self.geometry().center())
		self.about_program.move(current_pos.topLeft())
		self.about_program.show()
	
	def switchMode(self, action):
		if self.welcome_text.isVisible():
			self.welcome_text.setVisible(False)
			self.note_mess.setVisible(False)
			self.statusBar().setVisible(True)
		
		if action == self.sign_act:
			self.__runMode(self.sign)
		elif action == self.verify_sign_act:
			self.__runMode(self.verify_sign)
		elif action == self.encrypt_act:
			if self.encrypt.ability_visible:
				self.__runMode(self.encrypt)
			else:
				self.__runMode(self.mess)
		elif action == self.decrypt_act:
			self.__runMode(self.decrypt)
	
	def __runMode(self, v_mode):
		""" Install visible target mode and resize window if needed """
		
		map(lambda mode: mode.setVisible(True) if mode == v_mode else mode.setVisible(False), self.list_mode)
		try:
			v_mode.checkWindow()
		except: pass

class ThumbLoad(QThread):
	""" Thread thats will be converte the file to png """
	
	def __init__(self, parent = None):
		super(ThumbLoad, self).__init__(parent)
		
		self.parent = parent
		self.finished.connect(self.updateImage)
		self.__image_ready = False
	
	def run(self):
		self.path_name = self.parent.parent.toolTip().__str__().encode('utf-8')
		
		magic_obj = magic.open(magic.MAGIC_MIME) #created magic object whose return mimetype
		magic_obj.load() #load default databases data
		type_file = magic_obj.file(self.path_name)
		type_file = type_file[:type_file.find(';')] #remove entry 'charset'
		magic_obj.close()
		
		if 'pdf' in type_file:
			#maybe convert pdf to pdf through unoconv, but if file is large (~18Mb) then unoconv eating almost all RAM and not answered
			subprocess.call(['convert', '-density', '300', '-depth', '8', '-resize', '300', '-quality', '85', self.path_name+'[0]', OUTPUT_PNG]) #convert pdf to png
			self.__image_ready = True
		elif not [type_file for bad_type in ['video', 'audio', 'octet-stream', 'zip', 'tar'] if bad_type in type_file]:
			subprocess.call(['unoconv', '-e', 'PageRange=1-1', '-o', OUTPUT_PDF, '-f', 'pdf', self.path_name], stderr = DEVNULL) #convert 1 page document to pdf
			subprocess.call('convert -density 300 -depth 8 -resize 300 -quality 85 %s %s' %(OUTPUT_PDF, OUTPUT_PNG), shell=True) #convert pdf to png
			#convertation pdf to png could be done with python-wade (binding on ImageMagic),
			#but this package is not in repository and once use subprocess is not critical
			self.__image_ready = True
		else:
			self.__image_ready = None
	
	def updateImage(self):
		self.parent.thumb_show.updateThumb(self.__image_ready)

class ThumbShow(QFrame):
	""" Window to show thumbnail """
	
	def __init__(self, parent = None):
		super(ThumbShow, self).__init__(parent)
		
		self.setWindowFlags(Qt.ToolTip) #hidden head window
		self.setFrameStyle(QFrame.StyledPanel | QFrame.Raised) #set style
		
		self.wait_label = QLabel(_("Please Wait"), self)
		self.wait_label.setAlignment(Qt.AlignHCenter)
		self.wait_bar = QProgressBar(self)
		self.wait_bar.setRange(0, 0)
		self.wait_items = self.children() #create list wait widgets
		
		self.no_thumb = QLabel(_("Thumbnail of this file is not available yet")) #show message if thumbnail is impossible
		self.no_thumb.setAlignment(Qt.AlignHCenter)
		self.no_thumb.setVisible(False)
		
		self.vertical_layout = QVBoxLayout()
		self.vertical_layout.insertStretch(0, 1)
		self.vertical_layout.addWidget(self.no_thumb)
		self.vertical_layout.addWidget(self.wait_label)
		self.vertical_layout.addWidget(self.wait_bar)
		self.vertical_layout.insertStretch(-1, 1)
		
		self.setLayout(self.vertical_layout)
		
	def updateThumb(self, image_ready):
		""" Set thumbnail to background or set wait widgets """
		
		if image_ready:
			map(lambda item: item.setVisible(False), self.wait_items)
			self.no_thumb.setVisible(False)
			self.back_image = QPixmap(OUTPUT_PNG)
			self.setStyleSheet("background-image: url(%s); background-repeat: no-repeat" %OUTPUT_PNG)
		elif image_ready is None:
			self.setStyleSheet('')
			self.back_image = QPixmap(300, 400) #empty QPixmap
			map(lambda item: item.setVisible(False), self.wait_items)
			self.no_thumb.setVisible(True)
		else:
			self.setStyleSheet('')
			self.back_image = QPixmap(300, 400) #empty QPixmap
			map(lambda item: item.setVisible(True), self.wait_items)
			self.no_thumb.setVisible(False)
		
		self.resize(self.back_image.size())

class ThumbButton(QPushButton):
	""" Special button for show thumbnail """
	
	def __init__(self, parent = None):
		super(ThumbButton, self).__init__(parent)
		
		self.normal_icon = QIcon(PREFIX16 + 'hide.svg')
		self.active_icon = QIcon(PREFIX16 + 'show.svg')
		
		self.parent = parent
		self.save_tooltip = '' #nedded to restore tooltip
		
		self.setSizePolicy(QSizePolicy.Maximum, QSizePolicy.Fixed)
		self.setLayoutDirection(Qt.RightToLeft)
		self.setFlat(True)
		self.setStyleSheet("border: 0px")
		#set Icon
		self.setIcon(self.normal_icon)
		self.setIconSize(QSize(16, 16))
		#default button not seeing
		self.setVisible(False)
		
		self.thumb_show = ThumbShow()
	
	def enterEvent(self, QEvent):
		""" Handle event if mouse hover on button """
		
		self.save_tooltip = self.parent.toolTip() #saved tooltip
		self.parent.setToolTip('') #temporary remove tooltip that it was not displayed
		self.setIcon(self.active_icon)
		self.thumb_show.move(QCursor.pos().x()+20, QCursor.pos().y()-self.thumb_show.back_image.height()/2)
		self.thumb_show.show()
	
	def leaveEvent(self, QEvent):
		""" Handle event if mouse left the area of the button """
		
		self.setIcon(self.normal_icon)
		self.parent.setToolTip(self.save_tooltip)
		self.thumb_show.close()
	
	def mousePressEvent(self, QEvent):
		""" Handle event if mouse button press """
		
		self.thumb_show.close()
		self.move(self.pos().x()+1, self.pos().y()+1) #imitation button pressed, signal clicked do not work out
	
	def mouseReleaseEvent(self, QEvent):
		""" Handle event if mouse button release """
		
		self.setIcon(self.normal_icon)
		QDesktopServices.openUrl(QUrl.fromLocalFile(self.save_tooltip)) #open file in default programm

class AbstrFrame(QFrame):
	""" Basic widget for modes with CryptoPro
	Function showMode, resizeEvent, checkWindow
	responsible for right resize main window
	"""
	
	def __init__(self, parent = None):
		super(AbstrFrame, self).__init__(parent)
		
		self.parent = parent.parentWidget()
		self.setVisible(False)
		
		self.filter_file = _("All(*)")
		
		self.break_recursion = False
		self.ability_visible = True #sets the ability to force disable display of the form when switching to it
		self.elided_text = QFontMetrics(QFont()) #special tex (add ... if field small)
		
		self.list_off_frame = [] #disable/enable components section
		
		self.thumb_use = False #flag to access function to thumbnail
		
		#Labels
		label_file = QLabel(_("File:"))
		details = QLabel(_("Result:"))
		self.cert = QLabel(_("Certificate:"))
		self.mes_status_oper = QLabel()
		self.mes_status_oper.setFixedHeight(MAX_ROW_RESULT)
		self.old_mes_status_oper = "" #nedeed for dynamic elided text
		self.mes_status_oper.setWordWrap(True)
		self.icon_status_oper = QLabel()
		
		self.icon_success = QIcon(PREFIX16 + 'success.svg').pixmap(22,22)
		self.icon_atten = QIcon(PREFIX16 + 'attention.svg').pixmap(22,22)
		self.icon_failed = QIcon(PREFIX16 + 'fail.svg').pixmap(22,22)
		
		#Fields Edit
		self.file_edit = QLineEdit(readOnly = True)
		self.result = QTextEdit(readOnly = True)
		self.result.setVisible(False)
		
		#Buttons
		self.btn_open = QPushButton(self.parent.open_act.icon(), _("Choose"))
		self.btn_open.setToolTip(self.parent.open_act.toolTip())
		#if not set right parent then widgets will be set in wrong places
		self.btn_thumbnail = ThumbButton(self.file_edit)
		vbox_thumbnail = QVBoxLayout(self.file_edit)
		vbox_thumbnail.addWidget(self.btn_thumbnail)
		vbox_thumbnail.setContentsMargins(0,0,5,0)
		
		#Thread load thumbnail
		self.thumb_load = ThumbLoad(self.btn_thumbnail) #thread to create and load image
		
		self.btn_details = QPushButton(_("Details") + "  " u'\u25BC')
		self.btn_details.setToolTip(_("Show or hide more information"))
		self.btn_details.setEnabled(False)
		
		self.btn_details.clicked.connect(self.showMore)
		self.btn_open.clicked.connect(self.openFile)
		
		#Grid
		self.grid = QGridLayout()
		self.grid.setSpacing(5)
		
		self.grid.addWidget(label_file, 0, 0)
		self.grid.addWidget(self.file_edit, 0, 1)
		self.grid.addWidget(self.btn_open, 0, 2)
		
		self.grid.addWidget(details, 2, 0)
		hbox_status = QHBoxLayout()
		hbox_status.addWidget(self.icon_status_oper)
		hbox_status.addWidget(self.mes_status_oper, 1)
		self.grid.addLayout(hbox_status, 2, 1)
		self.grid.addWidget(self.result, 2, 1, -1, 1)
		self.grid.addWidget(self.btn_details, 2, 2)
		
		self.grid.setRowMinimumHeight(1, int((MIN_HEIGHT-self.parent.toolbar_height)*0.4))
		self.grid.setColumnMinimumWidth(0, self.cert.sizeHint().width())
		self.grid.setColumnMinimumWidth(2, 180)
		self.grid.setRowStretch(3, 1)
		self.grid.setSizeConstraint(QLayout.SetMinAndMaxSize)
		
		self.setLayout(self.grid)
	
	def showMore(self):
		""" Change height window and show/hide sub field """
		
		if self.result.isVisible():
			self.btn_details.setText(_("Details") + "  " u'\u25BC')
			self.result.setVisible(False)
			self.parent.setFixedHeight(MIN_HEIGHT)
		else:
			self.btn_details.setText(_("Details") + "  " u'\u25B2')
			self.result.setVisible(True)
			self.parent.setFixedHeight(MAX_HEIGHT)
		
	
	def openFile(self):
		""" Open file """
		
		dark_path_name = QFileDialog.getOpenFileName(self, _("Choose file"), os.path.expanduser("~"), self.filter_file)
		#if dialog "open file" was opened and closed without selecting file - nothing will happen
		#therefore variable self.path_name overwrites only if dark_path_name is not empty
		if dark_path_name:
			self.path_name = dark_path_name
			map(lambda item: item.setEnabled(True), self.list_off_frame)
			if self.result.isVisible():
				self.showMore()
			
			#parse path_name
			self.file_name = os.path.basename(self.path_name.__str__())
			self.currently_dir = os.path.dirname(self.path_name.__str__())
			
			self.file_edit.setText(self.file_name)
			self.file_edit.setToolTip(self.path_name)
			
			#set access to show thumbnail if thumb_use true
			if self.thumb_use:
				#stop thread if will be opened new file
				if self.thumb_load.isRunning():
					self.thumb_load.terminate()
					self.thumb_load.wait()
				self.btn_thumbnail.setVisible(True)
				self.btn_thumbnail.thumb_show.updateThumb(False)
				self.thumb_load.start()
			
			#clear field
			if self.btn_details.isEnabled(): self.btn_details.setEnabled(False)
			self.icon_status_oper.clear()
			self.mes_status_oper.clear()
			self.result.clear()
	
	def resizeEvent(self, e):
		""" Check state window and resize him (max to min) """
		
		if not self.parent.isMaximized():
			if self.result.isVisible():
				self.parent.resize(QSize(self.parent.width(), MAX_HEIGHT))
			else:
				self.parent.resize(QSize(self.parent.width(), MIN_HEIGHT))
		
		self.mes_status_oper.setText(self.elided_text.elidedText(self.old_mes_status_oper, Qt.ElideRight, self.parent.width(), Qt.TextWordWrap))
	
	def checkWindow(self):
		""" Resize window for each mode """
		
		if self.result.isVisible():
			self.parent.setFixedHeight(MAX_HEIGHT)
		else:
			self.parent.setFixedHeight(MIN_HEIGHT)

class SignFrame(AbstrFrame):
	""" Widget for create signature """
	
	def __init__(self, parent = None):
		super(SignFrame, self).__init__(parent)
		
		self.btn_sign = QPushButton(self.parent.sign_act.icon(), _("Sign file"))
		self.btn_sign.setToolTip(self.parent.sign_act.toolTip())
		
		self.thumb_use = True #enable thumbnail
		
		self.list_avail_cert = QComboBox(self)
		
		self.list_off_frame = [self.btn_sign, self.list_avail_cert]
		map(lambda item: item.setEnabled(False), self.list_off_frame)
		
		self.grid.addWidget(self.cert, 1, 0)
		self.grid.addWidget(self.list_avail_cert, 1, 1)
		self.grid.addWidget(self.btn_sign, 1, 2)

class VerifyFrame(AbstrFrame):
	""" Widget for verify signature """
	
	def __init__(self, parent = None):
		super(VerifyFrame, self).__init__(parent)
		
		self.filter_file = _("Signature file(*.sig)") + ";;" + self.filter_file
		
		self.btn_verify_sign = QPushButton(self.parent.verify_sign_act.icon(), _("Check signature"))
		self.btn_verify_sign.setToolTip(self.parent.verify_sign_act.toolTip())
		
		self.verify_split = QCheckBox(_("Split file"))
		self.verify_split.setToolTip(_("Separate the source file from the file signature"))
		self.verify_install = QCheckBox(_("Install certificate of the signature file"))
		self.verify_install.setToolTip(_("Install certificate in the local storage of the signature file"))
		
		vbox_verify_sig = QVBoxLayout()
		vbox_verify_sig.addWidget(self.verify_split)
		vbox_verify_sig.addWidget(self.verify_install)
		
		self.list_off_frame = [self.btn_verify_sign, self.verify_split, self.verify_install]
		map(lambda item: item.setEnabled(False), self.list_off_frame)
		
		self.grid.addLayout(vbox_verify_sig, 1, 1)
		self.grid.addWidget(self.btn_verify_sign, 1, 2)

class EncryptFrame(AbstrFrame):
	""" Widget for encrypted file """
	
	def __init__(self, parent = None):
		super(EncryptFrame, self).__init__(parent)
		
		self.list_avail_cert = QComboBox(self)
		self.btn_encrypt = QPushButton(self.parent.encrypt_act.icon(), self.parent.encrypt_act.text())
		self.btn_encrypt.setToolTip(self.parent.encrypt_act.toolTip())
		
		self.thumb_use = True #enable thumbnail
		
		self.list_off_frame = [self.btn_encrypt, self.list_avail_cert]
		map(lambda item: item.setEnabled(False), self.list_off_frame)
		
		self.grid.addWidget(self.cert, 1, 0)
		self.grid.addWidget(self.list_avail_cert, 1, 1)
		self.grid.addWidget(self.btn_encrypt, 1, 2)

class DescryptFrame(AbstrFrame):
	""" Widget for decrypted file """
	
	def __init__(self, parent = None):
		super(DescryptFrame, self).__init__(parent)
		
		self.filter_file = _("Encrypted file(*.enc)") + ";;" + self.filter_file
		
		self.list_avail_cert = QComboBox(self)
		self.btn_decrypt = QPushButton(self.parent.decrypt_act.icon(), self.parent.decrypt_act.text())
		self.btn_decrypt.setToolTip(self.parent.decrypt_act.toolTip())
		
		self.list_off_frame = [self.btn_decrypt, self.list_avail_cert]
		map(lambda item: item.setEnabled(False), self.list_off_frame)
		
		self.grid.addWidget(self.cert, 1, 0)
		self.grid.addWidget(self.list_avail_cert, 1, 1)
		self.grid.addWidget(self.btn_decrypt, 1, 2)

class MessFrame(QFrame):
	""" Frame for display global messages """
	
	def __init__(self, parent = None):
		super(MessFrame, self).__init__(parent)
		
		self.parent = parent.parentWidget()
		self.setVisible(False)
		
		self.cert_mess = QLabel(_("On this computer there is no certificate.\n\
			It can be set to check if the signature file at \"Check signature\" by clicking the option\n\
			and/or signing a file using the token mode \"Sign file\"."), self)
		self.cert_mess.setAlignment(Qt.AlignCenter)
		mess_vbox = QVBoxLayout()
		mess_vbox.addWidget(self.cert_mess)
		self.setLayout(mess_vbox)

class AboutWidget(QWidget):
	""" Widget about program """
	
	def __init__(self, parent = None):
		super(AboutWidget, self).__init__(parent)
		
		self.parent = parent
		
		rosa_logo = QIcon(PREFIX48 + 'rosa-crypto-tool.svg').pixmap(48,48)
		logoLabel = QLabel()
		logoLabel.setPixmap(rosa_logo)
		
		messages = QLabel()
		messages.setText(_("ROSA Crypto Tool - program for working with electronic digital signature\n\n"
						"Authors: Michl Voznesensky\n" +\
						"Version: 1.0.0\n" +\
						"License: BSD\n" +\
						"Copyright (c) 2016-2018, LLC \"NTC IT ROSA\""))
		
		grid = QGridLayout()
		grid.setSpacing(5)
		grid.addWidget(logoLabel, 0, 0, Qt.AlignCenter)
		grid.addWidget(messages, 1, 0, Qt.AlignCenter)
		
		self.setLayout(grid)
		
		self.resize(250, 150) #widht and hight window
		
		self.setWindowTitle(_("About ROSA Crypto Tool"))
		self.setWindowIcon(QIcon(PREFIX16 + 'about'))

if __name__ == '__main__':
	
	print("You run the part program rosa crypto tool, please launch the main file")
	
	#for debugger UI
	#
	import sys
	import gettext
	from PyQt5.QtGui import QApplication
	
	PREFIX16 ='../icon/16x16/' #directory icons
	PREFIX32 ='../icon/32x32/'
	PREFIX48 ='../icon/48x48/'
	
	gettext.install('rosa-crypto-tool')
	app = QApplication(sys.argv)
	app_ui = MainWindow()
	sys.exit(app.exec_())
	
